import { AttributeType, gl } from '@antv/l7-core';
import BaseLayer from '../core/BaseLayer';
import { LineTriangulation } from '../core/triangulation';
const line_dash_frag = "uniform float u_blur : 0.9;\nuniform float u_opacity : 1.0;\nuniform float u_dash_offset : 0.0;\nuniform float u_dash_ratio : 0.1;\nvarying vec4 v_color;\nvarying vec2 v_normal;\n\nuniform float u_time;\nuniform vec3 u_aimate: [ 0, 2., 1.0, 0.2 ];\n\nvarying float v_distance_ratio;\nvarying vec2 v_dash_array;\nvoid main() {\n   gl_FragColor = v_color;\n  //  gl_FragColor.a = v_distance_ratio;\n  // anti-alias\n  float blur = 1.- smoothstep(u_blur, 1., length(v_normal.xy)) * u_opacity;\n  // gl_FragColor.a *= blur  * ceil(mod(v_distance_ratio, v_dash_array.x) - v_dash_array.y);\n  gl_FragColor.a *= blur  * (1.0- step(v_dash_array.x, mod(v_distance_ratio, v_dash_array.x +v_dash_array.y)));\n\n\n}\n";
const line_dash_vert = "\nattribute float a_Miter;\nattribute vec4 a_Color;\nattribute float a_Size;\nattribute vec3 a_Normal;\nattribute float a_Total_Distance;\nattribute vec3 a_Position;\nattribute float a_Distance;\nuniform mat4 u_ModelMatrix;\nuniform vec2 u_dash_array: [10.0, 5.];\nuniform float u_dash_offset: 0;\n\nvarying vec4 v_color;\nvarying vec2 v_dash_array;\nvarying vec2 v_normal;\n\nvarying float v_distance_ratio;\n\n\n\n#pragma include \"projection\"\nvoid main() {\n\n  v_distance_ratio = a_Distance / a_Total_Distance;\n\n  v_dash_array = pow(2.0, 20.0 - u_Zoom) * u_dash_array / a_Total_Distance;\n\n  v_normal = vec2(reverse_offset_normal(a_Normal) * sign(a_Miter));\n  v_color = a_Color;\n  vec3 size = a_Miter * a_Size * reverse_offset_normal(a_Normal);  //v_normal * vec3(1., -1., 1.0);\n  vec2 offset = project_pixel(size.xy);\n  vec4 project_pos = project_position(vec4(a_Position.xy, 0, 1.0));\n  gl_Position = project_common_position_to_clipspace(vec4(project_pos.xy + offset, 0, 1.0));\n}\n";
export default class DashLineLayer extends BaseLayer {
  constructor(...args) {
    super(...args);
    this.name = 'LineLayer';
  }

  getConfigSchema() {
    return {
      properties: {
        opacity: {
          type: 'number',
          minimum: 0,
          maximum: 1
        }
      }
    };
  }

  renderModels() {
    const {
      opacity,
      dashArray = [10, 5]
    } = this.getLayerConfig();
    this.models.forEach(model => model.draw({
      uniforms: {
        u_opacity: opacity || 1.0,
        u_dash_array: dashArray
      }
    }));
    return this;
  }

  buildModels() {
    this.registerBuiltinAttributes();
    this.models = [this.buildLayerModel({
      moduleName: 'line_dash',
      vertexShader: line_dash_vert,
      fragmentShader: line_dash_frag,
      triangulation: LineTriangulation,
      blend: {
        enable: true,
        func: {
          srcRGB: gl.SRC_ALPHA,
          srcAlpha: 1,
          dstRGB: gl.ONE_MINUS_SRC_ALPHA,
          dstAlpha: 1
        }
      }
    })];
  }

  registerBuiltinAttributes() {
    this.styleAttributeService.registerStyleAttribute({
      name: 'size',
      type: AttributeType.Attribute,
      descriptor: {
        name: 'a_Size',
        buffer: {
          usage: gl.DYNAMIC_DRAW,
          data: [],
          type: gl.FLOAT
        },
        size: 1,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const {
            size
          } = feature;
          return Array.isArray(size) ? [size[0]] : [size];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: 'normal',
      type: AttributeType.Attribute,
      descriptor: {
        name: 'a_Normal',
        buffer: {
          usage: gl.STATIC_DRAW,
          data: [],
          type: gl.FLOAT
        },
        size: 3,
        update: (feature, featureIdx, vertex, attributeIdx, normal) => {
          return normal;
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: 'miter',
      type: AttributeType.Attribute,
      descriptor: {
        name: 'a_Miter',
        buffer: {
          usage: gl.DYNAMIC_DRAW,
          data: [],
          type: gl.FLOAT
        },
        size: 1,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          return [vertex[4]];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: 'startPos',
      type: AttributeType.Attribute,
      descriptor: {
        name: 'a_StartPos',
        buffer: {
          usage: gl.DYNAMIC_DRAW,
          data: [],
          type: gl.FLOAT
        },
        size: 3,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const coordinates = feature.coordinates;
          const coord = coordinates[0];
          return coord.length === 3 ? coord : [...coord, 0.0];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: 'distance',
      type: AttributeType.Attribute,
      descriptor: {
        name: 'a_Distance',
        buffer: {
          usage: gl.DYNAMIC_DRAW,
          data: [],
          type: gl.FLOAT
        },
        size: 1,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          return [vertex[3]];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: 'total_distance',
      type: AttributeType.Attribute,
      descriptor: {
        name: 'a_Total_Distance',
        buffer: {
          usage: gl.DYNAMIC_DRAW,
          data: [],
          type: gl.FLOAT
        },
        size: 1,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          return [vertex[5]];
        }
      }
    });
  }

}
//# sourceMappingURL=dash.js.map