"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.PointFillTriangulation = PointFillTriangulation;
exports.PointExtrudeTriangulation = PointExtrudeTriangulation;
exports.PointImageTriangulation = PointImageTriangulation;
exports.LineTriangulation = LineTriangulation;
exports.polygonTriangulation = polygonTriangulation;
exports.PolygonExtrudeTriangulation = PolygonExtrudeTriangulation;
exports.HeatmapGridTriangulation = HeatmapGridTriangulation;
exports.RasterImageTriangulation = RasterImageTriangulation;
exports.LineArcTriangulation = LineArcTriangulation;
exports.HeatmapTriangulation = HeatmapTriangulation;

var _l7Utils = require("@antv/l7-utils");

var _earcut = _interopRequireDefault(require("earcut"));

var _glMatrix = require("gl-matrix");

var _polylineNormal = _interopRequireDefault(require("../utils/polylineNormal"));

var _extrude = _interopRequireWildcard(require("./shape/extrude"));

var _Path = require("./shape/Path");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const GeometryCache = {};

function PointFillTriangulation(feature) {
  const coordinates = feature.coordinates;
  return {
    vertices: [...coordinates, ...coordinates, ...coordinates, ...coordinates],
    indices: [0, 1, 2, 2, 3, 0],
    size: coordinates.length
  };
}

function PointExtrudeTriangulation(feature) {
  const {
    shape
  } = feature;
  const {
    positions,
    index,
    normals
  } = getGeometry(shape, false);
  return {
    vertices: positions,
    indices: index,
    normals,
    size: 3
  };
}

function PointImageTriangulation(feature) {
  const coordinates = feature.coordinates;
  return {
    vertices: [...coordinates],
    indices: [0],
    size: coordinates.length
  };
}

function LineTriangulation(feature) {
  const {
    coordinates
  } = feature;
  let path = coordinates;

  if (Array.isArray(path[0][0])) {
    path = coordinates[0];
  }

  const line = (0, _polylineNormal.default)(path, false, 0);
  return {
    vertices: line.attrPos,
    indices: line.attrIndex,
    normals: line.normals,
    size: 6
  };
}

function polygonTriangulation(feature) {
  const {
    coordinates
  } = feature;

  const flattengeo = _earcut.default.flatten(coordinates);

  const {
    vertices,
    dimensions,
    holes
  } = flattengeo;
  return {
    indices: (0, _earcut.default)(vertices, holes, dimensions),
    vertices,
    size: dimensions
  };
}

function PolygonExtrudeTriangulation(feature) {
  const coordinates = feature.coordinates;
  const {
    positions,
    index,
    normals
  } = (0, _extrude.extrude_PolygonNormal)(coordinates, true);
  return {
    vertices: positions,
    indices: index,
    normals,
    size: 3
  };
}

function HeatmapGridTriangulation(feature) {
  const {
    shape
  } = feature;
  const {
    positions,
    index
  } = getHeatmapGeometry(shape);
  return {
    vertices: positions,
    indices: index,
    size: 3
  };
}

function RasterImageTriangulation(feature) {
  const coordinates = feature.coordinates;
  const positions = [...coordinates[0], 0, 0, 1, coordinates[1][0], coordinates[0][1], 0, 1, 1, ...coordinates[1], 0, 1, 0, ...coordinates[0], 0, 0, 1, ...coordinates[1], 0, 1, 0, coordinates[0][0], coordinates[1][1], 0, 0, 0];
  const indexs = [0, 1, 2, 3, 4, 5];
  return {
    vertices: positions,
    indices: indexs,
    size: 5
  };
}

function LineArcTriangulation(feature) {
  const segNum = 30;
  const coordinates = feature.coordinates;
  const positions = [];
  const indexArray = [];

  for (let i = 0; i < segNum; i++) {
    positions.push(i, 1, i, coordinates[0][0], coordinates[0][1], coordinates[1][0], coordinates[1][1], i, -1, i, coordinates[0][0], coordinates[0][1], coordinates[1][0], coordinates[1][1]);

    if (i !== segNum - 1) {
      indexArray.push(...[0, 1, 2, 1, 3, 2].map(v => {
        return i * 2 + v;
      }));
    }
  }

  return {
    vertices: positions,
    indices: indexArray,
    size: 7
  };
}

function HeatmapTriangulation(feature) {
  const coordinates = feature.coordinates;

  if (coordinates.length === 2) {
    coordinates.push(0);
  }

  const size = feature.size;
  const dir = addDir(-1, 1);
  const dir1 = addDir(1, 1);
  const dir2 = addDir(-1, -1);
  const dir3 = addDir(1, -1);
  const positions = [...coordinates, ...dir, ...coordinates, ...dir2, ...coordinates, ...dir3, ...coordinates, ...dir1];
  const indexArray = [0, 1, 2, 3, 0, 2];
  return {
    vertices: positions,
    indices: indexArray,
    size: 5
  };
}

function getGeometry(shape, needFlat = false) {
  if (GeometryCache && GeometryCache[shape]) {
    return GeometryCache[shape];
  }

  const path = _Path.geometryShape[shape] ? _Path.geometryShape[shape]() : _Path.geometryShape.cylinder();
  const geometry = (0, _extrude.extrude_PolygonNormal)([path], needFlat);
  GeometryCache[shape] = geometry;
  return geometry;
}

function computeVertexNormals(positions, indexArray, dim = 3, needFlat = false) {
  const normals = new Float32Array(positions.length / dim * 3);
  let vA;
  let vB;
  let vC;

  const cb = _glMatrix.vec3.create();

  const ab = _glMatrix.vec3.create();

  const normal = _glMatrix.vec3.create();

  for (let i = 0, li = indexArray.length; i < li; i += 3) {
    vA = indexArray[i + 0] * 3;
    vB = indexArray[i + 1] * 3;
    vC = indexArray[i + 2] * 3;
    let p1 = [positions[vA], positions[vA + 1]];
    let p2 = [positions[vB], positions[vB + 1]];
    let p3 = [positions[vC], positions[vC + 1]];

    if (needFlat) {
      p1 = (0, _l7Utils.lngLatToMeters)(p1);
      p2 = (0, _l7Utils.lngLatToMeters)(p2);
      p3 = (0, _l7Utils.lngLatToMeters)(p3);
    }

    const [ax, ay] = p1;

    const pA = _glMatrix.vec3.fromValues(ax, ay, positions[vA + 2]);

    const [bx, by] = p2;

    const pB = _glMatrix.vec3.fromValues(bx, by, positions[vB + 2]);

    const [cx, cy] = p3;

    const pC = _glMatrix.vec3.fromValues(cx, cy, positions[vC + 2]);

    _glMatrix.vec3.sub(cb, pC, pB);

    _glMatrix.vec3.sub(ab, pA, pB);

    _glMatrix.vec3.cross(normal, cb, ab);

    normals[vA] += cb[0];
    normals[vA + 1] += cb[1];
    normals[vA + 2] += cb[2];
    normals[vB] += cb[0];
    normals[vB + 1] += cb[1];
    normals[vB + 2] += cb[2];
    normals[vC] += cb[0];
    normals[vC + 1] += cb[1];
    normals[vC + 2] += cb[2];
  }

  normalizeNormals(normals);
  return normals;
}

function normalizeNormals(normals) {
  for (let i = 0, li = normals.length; i < li; i += 3) {
    const normal = _glMatrix.vec3.fromValues(normals[i], normals[i + 1], normals[i + 2]);

    const newNormal = _glMatrix.vec3.create();

    _glMatrix.vec3.normalize(newNormal, normal);

    normals.set(newNormal, i);
  }
}

function checkIsClosed(points) {
  const p1 = points[0][0];
  const p2 = points[0][points[0].length - 1];
  return p1[0] === p2[0] && p1[1] === p2[1];
}

function getHeatmapGeometry(shape) {
  const shape3d = ['cylinder', 'triangleColumn', 'hexagonColumn', 'squareColumn'];
  const path = _Path.geometryShape[shape] ? _Path.geometryShape[shape]() : _Path.geometryShape.circle();
  const geometry = shape3d.indexOf(shape) === -1 ? (0, _extrude.fillPolygon)([path]) : (0, _extrude.default)([path]);
  return geometry;
}

function addDir(dirX, dirY) {
  const x = (dirX + 1) / 2;
  const y = (dirY + 1) / 2;
  return [x, y];
}
//# sourceMappingURL=triangulation.js.map