"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.PointTriangulation = PointTriangulation;
exports.default = void 0;

var _l7Core = require("@antv/l7-core");

var _BaseLayer = _interopRequireDefault(require("../core/BaseLayer"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const textFrag = "uniform sampler2D u_sdf_map;\nuniform float u_gamma_scale : 0.5;\nuniform float u_font_size : 24;\nuniform float u_opacity : 1.0;\nuniform vec4 u_stroke : [0, 0, 0, 1];\nuniform float u_strokeWidth : 2.0;\nuniform float u_halo_blur : 0.5;\n\nvarying vec4 v_color;\nvarying vec2 v_uv;\nvarying float v_gamma_scale;\n\nvoid main() {\n  // get sdf from atlas\n  float dist = texture2D(u_sdf_map, v_uv).a;\n\n  float fontScale = u_font_size / 24.0;\n\n  lowp float buff = (6.0 - u_strokeWidth / fontScale) / SDF_PX;\n  highp float gamma = (u_halo_blur * 1.19 / SDF_PX + EDGE_GAMMA) / (fontScale * u_gamma_scale);\n\n  highp float gamma_scaled = gamma * v_gamma_scale;\n\n  highp float alpha = smoothstep(buff - gamma_scaled, buff + gamma_scaled, dist);\n\n  gl_FragColor = mix(v_color * u_opacity, u_stroke, smoothstep(0., 0.5, 1. - dist)) * alpha;\n}\n";
const textVert = "attribute vec3 a_Position;\nattribute vec2 a_tex;\nattribute vec2 a_offset;\nattribute vec4 a_color;\nattribute float a_size;\n\nuniform vec2 u_sdf_map_size;\nuniform vec2 u_viewport_size;\n\nuniform float u_activeId : 0;\nuniform vec4 u_activeColor : [1.0, 0.0, 0.0, 1.0];\n\nvarying vec2 v_uv;\nvarying float v_gamma_scale;\nvarying vec4 v_color;\n\n#pragma include \"projection\"\n\nvoid main() {\n  v_color = a_color;\n  v_uv = a_tex / u_sdf_map_size;\n\n  // \u6587\u672C\u7F29\u653E\u6BD4\u4F8B\n  float fontScale = a_size / 24.;\n\n  vec4 project_pos = project_position(vec4(a_Position, 1.0));\n\n  vec4 projected_position  = project_common_position_to_clipspace(vec4(project_pos.xyz, 1.0));\n\n  gl_Position = vec4(projected_position.xy / projected_position.w\n    + a_offset * fontScale / u_viewport_size * 2., 0.0, 1.0);\n  v_gamma_scale = gl_Position.w;\n\n\n}\n";

function PointTriangulation(feature) {
  const coordinates = feature.coordinates;
  return {
    vertices: [...coordinates, ...coordinates, ...coordinates, ...coordinates],
    indices: [0, 1, 2, 2, 3, 0],
    size: coordinates.length
  };
}

class TextLayer extends _BaseLayer.default {
  constructor(...args) {
    super(...args);
    this.name = 'PointLayer';
  }

  getConfigSchema() {
    return {
      properties: {
        opacity: {
          type: 'number',
          minimum: 0,
          maximum: 1
        }
      }
    };
  }

  renderModels() {
    const {
      opacity
    } = this.getLayerConfig();
    this.models.forEach(model => model.draw({
      uniforms: {
        u_opacity: opacity || 1.0
      }
    }));
    return this;
  }

  buildModels() {
    this.registerBuiltinAttributes();
    this.models = [this.buildLayerModel({
      moduleName: 'pointText',
      vertexShader: textVert,
      fragmentShader: textFrag,
      triangulation: PointTriangulation,
      depth: {
        enable: false
      },
      blend: {
        enable: true,
        func: {
          srcRGB: _l7Core.gl.SRC_ALPHA,
          srcAlpha: 1,
          dstRGB: _l7Core.gl.ONE_MINUS_SRC_ALPHA,
          dstAlpha: 1
        }
      }
    })];
  }

  registerBuiltinAttributes() {
    this.styleAttributeService.registerStyleAttribute({
      name: 'textOffsets',
      type: _l7Core.AttributeType.Attribute,
      descriptor: {
        name: 'a_textOffsets',
        buffer: {
          usage: _l7Core.gl.STATIC_DRAW,
          data: [],
          type: _l7Core.gl.FLOAT
        },
        size: 2,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const extrude = [-1, -1, 1, -1, 1, 1, -1, 1];
          const extrudeIndex = attributeIdx % 4 * 2;
          return [extrude[extrudeIndex], extrude[extrudeIndex + 1]];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: 'size',
      type: _l7Core.AttributeType.Attribute,
      descriptor: {
        name: 'a_Size',
        buffer: {
          usage: _l7Core.gl.DYNAMIC_DRAW,
          data: [],
          type: _l7Core.gl.FLOAT
        },
        size: 1,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const {
            size
          } = feature;
          return Array.isArray(size) ? [size[0]] : [size];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: 'shape',
      type: _l7Core.AttributeType.Attribute,
      descriptor: {
        name: 'a_Shape',
        buffer: {
          usage: _l7Core.gl.DYNAMIC_DRAW,
          data: [],
          type: _l7Core.gl.FLOAT
        },
        size: 1,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const {
            shape = 2
          } = feature;
          const shape2d = this.getLayerConfig().shape2d;
          const shapeIndex = shape2d.indexOf(shape);
          return [shapeIndex];
        }
      }
    });
  }

  initTextFont() {
    const {
      fontWeight = 'normal',
      fontFamily
    } = this.getLayerConfig();
    const data = this.getEncodedData();
    const characterSet = [];
    data.forEach(item => {
      let {
        shape = ''
      } = item;
      shape = shape.toString();

      for (const char of shape) {
        if (characterSet.indexOf(char) === -1) {
          characterSet.push(char);
        }
      }
    });
    this.fontService.setFontOptions({
      characterSet,
      fontWeight,
      fontFamily
    });
  }

}

exports.default = TextLayer;
//# sourceMappingURL=text.js.map