"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _l7Core = require("@antv/l7-core");

var _BaseLayer = _interopRequireDefault(require("../core/BaseLayer"));

var _triangulation = require("../core/triangulation");

var _color = require("../utils/color");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const rasterImageFrag = "precision mediump float;\nuniform float u_opacity: 1.0;\nuniform sampler2D u_texture;\nuniform sampler2D u_colorTexture;\nuniform float u_min;\nuniform float u_max;\nvarying vec2 v_texCoord;\n\nvoid main() {\n\n  float value = texture2D(u_texture,vec2(v_texCoord.x,v_texCoord.y)).a;\n   value = clamp(value,u_min,u_max);\n  float value1 =  (value - u_min) / (u_max -u_min);\n  vec2 ramp_pos = vec2(\n        fract(16.0 * (1.0 - value1)),\n        floor(16.0 * (1.0 - value1)) / 16.0);\n  gl_FragColor =  texture2D(u_colorTexture,ramp_pos);;\n}\n";
const rasterImageVert = "precision highp float;\nuniform mat4 u_ModelMatrix;\nattribute vec3 a_Position;\nattribute vec2 a_Uv;\nvarying vec2 v_texCoord;\n#pragma include \"projection\"\nvoid main() {\n   v_texCoord = a_Uv;\n   vec4 project_pos = project_position(vec4(a_Position, 1.0));\n   gl_Position = project_common_position_to_clipspace(vec4(project_pos.xy,0., 1.0));\n}\n";

class Raster2dLayer extends _BaseLayer.default {
  constructor(...args) {
    super(...args);
    this.name = 'RasterLayer';
    this.texture = void 0;
    this.colorTexture = void 0;
  }

  getConfigSchema() {
    return {
      properties: {
        opacity: {
          type: 'number',
          minimum: 0,
          maximum: 1
        }
      }
    };
  }

  renderModels() {
    const {
      opacity
    } = this.getLayerConfig();
    const parserDataItem = this.getSource().data.dataArray[0];
    const {
      min,
      max
    } = parserDataItem;

    if (this.texture) {
      this.models.forEach(model => model.draw({
        uniforms: {
          u_opacity: opacity || 1,
          u_texture: this.texture,
          u_min: min,
          u_max: max,
          u_colorTexture: this.colorTexture
        }
      }));
    }

    return this;
  }

  buildModels() {
    this.registerBuiltinAttributes();
    const source = this.getSource();
    const {
      createTexture2D
    } = this.rendererService;
    const parserDataItem = this.getSource().data.dataArray[0];
    this.texture = createTexture2D({
      data: parserDataItem.data,
      width: parserDataItem.width,
      height: parserDataItem.height,
      format: _l7Core.gl.LUMINANCE,
      type: _l7Core.gl.FLOAT,
      aniso: 4
    });
    const {
      rampColors
    } = this.getLayerConfig();
    const imageData = (0, _color.generateColorRamp)(rampColors);
    this.colorTexture = createTexture2D({
      data: imageData.data,
      width: imageData.width,
      height: imageData.height,
      flipY: true
    });
    this.models = [this.buildLayerModel({
      moduleName: 'Raster3DImage',
      vertexShader: rasterImageVert,
      fragmentShader: rasterImageFrag,
      triangulation: _triangulation.RasterImageTriangulation,
      primitive: _l7Core.gl.TRIANGLES,
      depth: {
        enable: false
      },
      blend: {
        enable: true,
        func: {
          srcRGB: _l7Core.gl.SRC_ALPHA,
          srcAlpha: 1,
          dstRGB: _l7Core.gl.ONE_MINUS_SRC_ALPHA,
          dstAlpha: 1
        }
      }
    })];
  }

  registerBuiltinAttributes() {
    this.styleAttributeService.registerStyleAttribute({
      name: 'uv',
      type: _l7Core.AttributeType.Attribute,
      descriptor: {
        name: 'a_Uv',
        buffer: {
          usage: _l7Core.gl.DYNAMIC_DRAW,
          data: [],
          type: _l7Core.gl.FLOAT
        },
        size: 2,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          return [vertex[3], vertex[4]];
        }
      }
    });
  }

}

exports.default = Raster2dLayer;
//# sourceMappingURL=raster2d.js.map